# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Email Verification page UI."""

from twisted.internet import defer

from ubuntu_sso.qt import build_general_error_message
from ubuntu_sso.qt.gui import SSOWizardPage
from ubuntu_sso.logger import setup_logging
from ubuntu_sso.utils.ui import (
    VERIFY_EMAIL_TITLE,
    VERIFY_EMAIL_CONTENT,
)
from ubuntu_sso.utils.ui import ERROR_EMAIL_TOKEN


logger = setup_logging('ubuntu_sso.email_verification_page')


class EmailVerificationPage(SSOWizardPage):
    """Widget used to input the email verification code."""

    def __init__(self, ui, *args, **kwargs):
        super(EmailVerificationPage, self).__init__(ui, *args, **kwargs)
        self.email = ''
        self._signals = {
            'EmailValidated':
             self._filter_by_app_name(self.on_email_validated),
            'EmailValidationError':
             self._filter_by_app_name(self.on_email_validation_error),
        }
        self.setup_page()

    @defer.inlineCallbacks
    def setup_page(self):
        """Setup the ui components."""
        self.backend = yield self.get_backend()
        self._setup_signals()
        self._connect_ui_elements()

    @property
    def verification_code(self):
        """Return the content of the verification code edit."""
        return str(self.ui.verification_code_edit.text())

    @property
    def next_button(self):
        """Return the button that move to the next stage."""
        return self.ui.next_button

    def _connect_ui_elements(self):
        """Set the connection of signals."""
        logger.debug('EmailVerificationController._connect_ui_elements')
        self.ui.verification_code_edit.textChanged.connect(
            self.validate_form)
        self.next_button.clicked.connect(self.validate_email)

    def validate_form(self):
        """Check the state of the form."""
        code = self.verification_code.strip()
        enabled = len(code) > 0
        self.next_button.setEnabled(enabled)
        self.next_button.setProperty('DisabledState',
            not self.next_button.isEnabled())
        self.next_button.style().unpolish(
            self.next_button)
        self.next_button.style().polish(
            self.next_button)

    def _set_titles(self):
        """Set the different titles."""
        logger.debug('EmailVerificationController._set_titles')
        self.header.set_title(VERIFY_EMAIL_TITLE)
        self.header.set_subtitle(VERIFY_EMAIL_CONTENT % {
            "app_name": self.app_name,
            "email": self.email,
        })

    def set_titles(self, email):
        """This class needs to have a public set_titles.

        Since the subtitle contains data that is only known after SetupAccount
        and _set_titles is only called on initialization.
        """
        self.email = email
        self._set_titles()

    def validate_email(self):
        """Call the next action."""
        logger.debug('EmailVerificationController.validate_email')
        email = unicode(self.wizard().field('email_address').toString())
        password = unicode(self.wizard().field('password').toString())
        code = unicode(self.ui.verification_code_edit.text())
        self.backend.validate_email(self.app_name, email,
                                    password, code)

    def on_email_validated(self, app_name, *args, **kwargs):
        """Signal thrown after the email is validated."""
        logger.info('EmailVerificationController.on_email_validated')
        email = self.wizard().field('email_address').toString()
        self.wizard().registrationSuccess.emit(app_name, email)

    def on_email_validation_error(self, app_name, error):
        """Signal thrown when there's a problem validating the email."""
        msg = error.pop(ERROR_EMAIL_TOKEN, '')
        msg += build_general_error_message(error)
        self.message_box.critical(self, self.app_name, msg)

    #pylint: disable=C0103
    def initializePage(self):
        """Called to prepare the page just before it is shown."""
        self.next_button.setDefault(True)
        self.next_button.setEnabled(False)
        self.next_button.setProperty('DisabledState',
            not self.next_button.isEnabled())
        self.next_button.style().unpolish(self.next_button)
        self.next_button.style().polish(self.next_button)
    #pylint: enable=C0103
