# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Forgotten Password page UI."""

from twisted.internet import defer

from ubuntu_sso.qt.gui import SSOWizardEnhancedEditPage
from ubuntu_sso.utils.ui import (
    EMAIL_LABEL,
    is_correct_email,
    RESET_PASSWORD,
    REQUEST_PASSWORD_TOKEN_LABEL,
    REQUEST_PASSWORD_TOKEN_WRONG_EMAIL,
    REQUEST_PASSWORD_TOKEN_TECH_ERROR,
    TRY_AGAIN_BUTTON,
)


class ForgottenPasswordPage(SSOWizardEnhancedEditPage):
    """Widget used to deal with users that forgot the password."""

    def __init__(self, ui, *args, **kwargs):
        super(ForgottenPasswordPage, self).__init__(ui, *args, **kwargs)
        self._signals = {
            'PasswordResetTokenSent':
             self._filter_by_app_name(self.on_password_reset_token_sent),
            'PasswordResetError':
             self._filter_by_app_name(self.on_password_reset_error),
        }
        self.setup_page()

    @defer.inlineCallbacks
    def setup_page(self):
        """Setup the widget components."""
        self.backend = yield self.get_backend()
        self._setup_signals()
        # hide the error label
        self.try_again_widget.setVisible(False)
        self._set_translated_strings()
        self._connect_ui()
        self._set_enhanced_line_edit()
        self._register_fields()

    @property
    def email_widget(self):
        """Return the widget used to show the email information."""
        return self.ui.email_widget

    @property
    def forgotted_password_intro_label(self):
        """Return the intro label that lets the user know the issue."""
        return self.ui.forgotted_password_intro_label

    @property
    def error_label(self):
        """Return the label used to show error."""
        return self.ui.error_label

    @property
    def email_address_label(self):
        """Return the lable used to state the use of the line edit."""
        return self.ui.email_address_label

    @property
    def email_address(self):
        """Return the email address provided by the user."""
        return str(self.ui.email_line_edit.text())

    @property
    def email_address_line_edit(self):
        """Return the line edit with the content."""
        return self.ui.email_line_edit

    @property
    def send_button(self):
        """Return the button used to request the new password."""
        return self.ui.send_button

    @property
    def try_again_widget(self):
        """Return the widget used to display the try again button."""
        return self.ui.try_again_widget

    @property
    def try_again_button(self):
        """Return the button used to try again the reset password."""
        return self.ui.try_again_button

    #pylint: disable=C0103
    def initializePage(self):
        """Set the initial state of ForgottenPassword page."""
        self.send_button.setDefault(True)
        enabled = not self.ui.email_line_edit.text().isEmpty()
        self.send_button.setEnabled(enabled)
        # The style from this property come from the Wizard
        self.send_button.setProperty("DisabledState",
            not self.send_button.isEnabled())
        self.send_button.style().unpolish(self.send_button)
        self.send_button.style().polish(self.send_button)
    #pylint: enable=C0103

    def _register_fields(self):
        """Register the fields of the wizard page."""
        self.registerField('email_address',
                                self.email_address_line_edit)

    def _set_translated_strings(self):
        """Set the translated strings in the view."""
        self.forgotted_password_intro_label.setText(
                                    REQUEST_PASSWORD_TOKEN_LABEL % {'app_name':
                                    self.app_name})
        self.email_address_label.setText(EMAIL_LABEL)
        self.send_button.setText(RESET_PASSWORD)
        self.try_again_button.setText(TRY_AGAIN_BUTTON)

    def _set_enhanced_line_edit(self):
        """Set the extra logic to the line edits."""
        self.set_line_edit_validation_rule(
                                           self.email_address_line_edit,
                                           is_correct_email)

    def _connect_ui(self):
        """Connect the diff signals from the Ui."""
        self.email_address_line_edit.textChanged.connect(self._validate)
        self.send_button.clicked.connect(
                            lambda: self.backend.request_password_reset_token(
                                                self.app_name,
                                                self.email_address))
        self.try_again_button.clicked.connect(self.on_try_again)

    def _validate(self):
        """Validate that we have an email."""
        email = unicode(self.email_address_line_edit.text())
        self.send_button.setEnabled(is_correct_email(email))
        self.send_button.setProperty("DisabledState",
            not self.send_button.isEnabled())
        self.send_button.style().unpolish(self.send_button)
        self.send_button.style().polish(self.send_button)

    def on_try_again(self):
        """Set back the widget to the initial state."""
        self.try_again_widget.setVisible(False)
        self.email_widget.setVisible(True)

    # pylint: disable=W0212
    def on_password_reset_token_sent(self):
        """Action taken when we managed to get the password reset done."""
        # ignore the result and move to the reset page
        self.next = self.wizard()._pages[self.wizard().reset_password]
        self.wizard().next()
    # pylint: enable=W0212

    def on_password_reset_error(self, app_name, error):
        """Action taken when there was an error requesting the reset."""
        # set the error message
        msg = REQUEST_PASSWORD_TOKEN_TECH_ERROR
        if error['errtype'] == 'ResetPasswordTokenError':
            # the account provided is wrong, lets tell the user to try
            # again
            msg = REQUEST_PASSWORD_TOKEN_WRONG_EMAIL
        else:
            # ouch, I dont know what went wrong, tell the user to try later
            self.email_widget.setVisible(False)
            self.forgotted_password_intro_label.setVisible(False)
            self.try_again_widget.setVisible(True)
        self.message_box.critical(self, self.app_name, msg)
