# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Reset Password page UI."""

from PyQt4.QtCore import SIGNAL
from PyQt4.QtGui import QApplication
from twisted.internet import defer

from ubuntu_sso.qt import build_general_error_message
from ubuntu_sso.qt.gui import SSOWizardEnhancedEditPage
from ubuntu_sso.qt import common
from ubuntu_sso.utils.ui import (
    is_min_required_password,
    PASSWORD1_ENTRY,
    PASSWORD2_ENTRY,
    PASSWORD_HELP,
    RESET_CODE_ENTRY,
    RESET_PASSWORD,
    RESET_TITLE,
    RESET_SUBTITLE,
)
from ubuntu_sso.logger import setup_logging


logger = setup_logging('ubuntu_sso.reset_password_page')


class ResetPasswordPage(SSOWizardEnhancedEditPage):
    """Widget used to allow the user change his password."""

    def __init__(self, ui, app_name=None, parent=None):
        """Create a new instance."""
        super(ResetPasswordPage, self).__init__(ui, app_name, parent)
        self.ui.password_line_edit.textEdited.connect(
            lambda: common.password_assistance(self.ui.password_line_edit,
                                                 self.ui.password_assistance,
                                                 common.NORMAL))
        self.ui.confirm_password_line_edit.textEdited.connect(
            lambda: common.password_check_match(self.ui.password_line_edit,
                                      self.ui.confirm_password_line_edit,
                                      self.ui.password_assistance))
        self._signals = {
            'PasswordChanged':
             self._filter_by_app_name(self.on_password_changed),
            'PasswordChangeError':
             self._filter_by_app_name(self.on_password_change_error),
        }
        self.setup_page()

    @defer.inlineCallbacks
    def setup_page(self):
        """Setup the widget components."""
        self.backend = yield self.get_backend()
        self._setup_signals()
        self._set_translated_strings()
        self._connect_ui()
        self._add_line_edits_validations()

    def focus_changed(self, old, now):
        """Check who has the focus to activate password popups if necessary."""
        if now == self.ui.password_line_edit:
            self.ui.password_assistance.setVisible(True)
            common.password_default_assistance(self.ui.password_assistance)
        elif now == self.ui.confirm_password_line_edit:
            common.password_check_match(self.ui.password_line_edit,
                                      self.ui.confirm_password_line_edit,
                                      self.ui.password_assistance)

    # Invalid name "initializePage"
    # pylint: disable=C0103

    def initializePage(self):
        """Extends QWizardPage initializePage method."""
        super(ResetPasswordPage, self).initializePage()
        common.password_default_assistance(self.ui.password_assistance)
        self.ui.password_assistance.setVisible(False)
        self.setTitle(RESET_TITLE)
        self.setSubTitle(RESET_SUBTITLE)
        self.ui.password_label.setText(PASSWORD1_ENTRY)
        self.ui.confirm_password_label.setText(PASSWORD2_ENTRY)
        self.ui.reset_code.setText(RESET_CODE_ENTRY)

        self.ui.reset_password_button.setDefault(True)
        self.ui.reset_password_button.setEnabled(False)
        # The style from this property come from the Wizard
        self.ui.reset_password_button.setProperty("DisabledState",
            not self.ui.reset_password_button.isEnabled())
        self.ui.reset_password_button.style().unpolish(
            self.ui.reset_password_button)
        self.ui.reset_password_button.style().polish(
            self.ui.reset_password_button)

    def showEvent(self, event):
        """Connect focusChanged signal from the application."""
        super(ResetPasswordPage, self).showEvent(event)
        self.connect(QApplication.instance(),
            SIGNAL("focusChanged(QWidget*, QWidget*)"),
            self.focus_changed)

    def hideEvent(self, event):
        """Disconnect the focusChanged signal when the page change."""
        super(ResetPasswordPage, self).hideEvent(event)
        try:
            self.disconnect(QApplication.instance(),
                SIGNAL("focusChanged(QWidget*, QWidget*)"),
                self.focus_changed)
        except TypeError:
            pass

    # pylint: enable=C0103

    def _set_translated_strings(self):
        """Translate the diff strings used in the app."""
        self.ui.reset_password_button.setText(RESET_PASSWORD)
        self.setSubTitle(PASSWORD_HELP)

    def _connect_ui(self):
        """Connect the different ui signals."""
        self.ui.reset_password_button.clicked.connect(
                                                    self.set_new_password)
        self.ui.reset_code_line_edit.textChanged.connect(self._validate)
        self.ui.password_line_edit.textChanged.connect(self._validate)
        self.ui.confirm_password_line_edit.textChanged.connect(
            self._validate)

    def _validate(self):
        """Enable the submit button if data is valid."""
        enabled = True
        code = unicode(self.ui.reset_code_line_edit.text())
        password = unicode(self.ui.password_line_edit.text())
        confirm_password = unicode(
            self.ui.confirm_password_line_edit.text())
        if not is_min_required_password(password):
            enabled = False
        elif not self.is_correct_password_confirmation(confirm_password):
            enabled = False
        elif not code:
            enabled = False
        self.ui.reset_password_button.setEnabled(enabled)
        self.ui.reset_password_button.setProperty("DisabledState",
            not self.ui.reset_password_button.isEnabled())
        self.ui.reset_password_button.style().unpolish(
            self.ui.reset_password_button)
        self.ui.reset_password_button.style().polish(
            self.ui.reset_password_button)

    def _add_line_edits_validations(self):
        """Add the validations to be use by the line edits."""
        self.set_line_edit_validation_rule(
                                           self.ui.password_line_edit,
                                           is_min_required_password)
        self.set_line_edit_validation_rule(
                                    self.ui.confirm_password_line_edit,
                                    self.is_correct_password_confirmation)
        # same as the above case, lets connect a signal to a signal
        self.ui.password_line_edit.textChanged.connect(
                     self.ui.confirm_password_line_edit.textChanged.emit)

    def on_password_changed(self, app_name, result):
        """Let user know that the password was changed."""
        email = unicode(self.wizard().forgotten.ui.email_line_edit.text())
        self.wizard().current_user.ui.email_edit.setText(email)
        self.overlay.hide()
        current_user_id = self.wizard().current_user_page_id
        visited_pages = self.wizard().visitedPages()
        for index in reversed(visited_pages):
            if index == current_user_id:
                break
            self.wizard().back()

    def on_password_change_error(self, app_name, error):
        """Let the user know that there was an error."""
        logger.error('Got error changing password for %s, error: %s',
                     self.app_name, error)
        self.message_box.critical(self, self.app_name,
            build_general_error_message(error))

    def set_new_password(self):
        """Request a new password to be set."""
        email = unicode(self.wizard().forgotten.ui.email_line_edit.text())
        code = unicode(self.ui.reset_code_line_edit.text())
        password = unicode(self.ui.password_line_edit.text())
        logger.info('Setting new password for %r and email %r with code %r',
                    self.app_name, email, code)
        self.backend.set_new_password(self.app_name, email, code, password)

    def is_correct_password_confirmation(self, password):
        """Return if the password is correct."""
        return unicode(self.ui.password_line_edit.text()) == password
