# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test the Ui code."""

from PyQt4 import QtGui, QtCore
from twisted.internet import defer
from twisted.trial.unittest import TestCase
from ubuntu_sso.qt import loadingoverlay

from ubuntu_sso import main


class FakedObject(object):
    """Fake an object, record every call."""

    next_result = None
    exposed_methods = ['connect_to_signal',
        'generate_captcha']

    def __init__(self, *args, **kwargs):
        self._args = args
        self._kwargs = kwargs
        self._called = {}
        for i in self.exposed_methods:
            setattr(self, i, self._record_call(i))

    def _record_call(self, func_name):
        """Store values when calling 'func_name'."""

        def inner(*args, **kwargs):
            """Fake 'func_name'."""
            self._called[func_name] = (args, kwargs)
            return self.next_result

        return inner


class FakedBackend(object):
    """A faked backend."""

    sso_login = FakedObject()


class FakePageUiStyle(object):
    """Fake the page."""

    def __init__(self):
        self.ui = self
        self.properties = {}
        super(FakePageUiStyle, self).__init__()

    def wizard(self):
        """Use itself as a fake wizard, too."""
        return self

    def text(self):
        """Return text."""
        return self.properties.get('text', '')

    # setText, setEnabled are inherited
    # pylint: disable=C0103
    def setText(self, text):
        """Save text."""
        self.properties['text'] = text

    def setEnabled(self, value):
        """Fake setEnabled."""
        self.properties['enabled'] = value

    def enabled(self):
        """Fake enabled."""
        return self.properties.get('enabled', False)

    def isEnabled(self):
        """Fake isEnabled."""
        return self.properties.get('enabled', False)

    def setProperty(self, key, val):
        """Fake setProperty to restyle some widget."""
        self.properties[key] = val

    def property(self, key):
        """Fake property from widget style."""
        return self.properties.get(key, False)

    def setDefault(self, val):
        """Fake button setDefault."""
        self.properties['default'] = val

    def style(self):
        """Fake style."""
        return self

    def unpolish(self, *args):
        """Fake unpolish."""
        self.properties['unpolish'] = len(args) > 0

    def polish(self, *args):
        """Fake polish."""
        self.properties['polish'] = len(args) > 0

    def setVisible(self, val):
        """Fake setVisible from Qt."""
        self.properties['visible'] = val

    def isVisible(self):
        """Fake isVisible from Qt."""
        return self.properties['visible']


class FakeOverlay(object):

    """A fake delay overlay."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.show_counter = 0
        self.hide_counter = 0
        self.args = (args, kwargs)

    def show(self):
        """Fake show."""
        self.show_counter += 1

    def hide(self):
        """Fake hide."""
        self.hide_counter += 1

    def resize(self, *args):
        """Fake resize."""


class FakeSignal(object):

    """A fake PyQt signal."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.target = None

    def connect(self, target):
        """Fake connect."""
        self.target = target

    def disconnect(self, *args):
        """Fake disconnect."""
        self.target = None

    def emit(self, *args):
        """Fake emit."""
        if self.target:
            self.target(*args)


class FakeWizardPage(object):

    """A fake wizard page."""

    def __init__(self, *args, **kwargs):
        self.has_back_button = True

    # pylint: disable=C0103
    def initializePage(self):
        """Fake initializePage."""


class FakeMainWindow(object):

    """A fake MainWindow."""

    currentIdChanged = FakeSignal()
    loginSuccess = FakeSignal()
    registrationIncomplete = FakeSignal()
    registrationSuccess = FakeSignal()
    userCancellation = FakeSignal()
    shown = False
    SYNC_NOW_OR_LATER_PAGE = 4
    CONGRATULATIONS_PAGE = 5
    folders_page_id = 6
    local_folders_page_id = 7

    def __init__(self, close_callback=None):
        self.button_texts = []
        self.button_layout = None
        self.options = []
        self.overlay = FakeOverlay()
        self.local_folders_page = FakeWizardPage()
        self.folders_page = FakeWizardPage()
        self.app_name = 'app_name'

    def show(self):
        """Fake method."""
        self.shown = True

    # pylint: disable=C0103
    def setButtonText(self, button, text):
        """Fake setButtonText."""
        self.button_texts.append((button, text))

    def setButtonLayout(self, layout):
        """Fake setButtonText."""
        self.button_layout = layout

    def button(self, button):
        """Fake button method to obtain the wizard buttons."""
        return QtGui.QPushButton()

    def setOption(self, *args):
        """Fake setOption."""
        self.options.append(args)

    def next(self):
        """Fake next."""

    def reject(self):
        """Fake reject."""


class FakeController(object):

    """A fake controller for the tests."""

    is_fake = True
    set_next_validation = lambda *a, **kw: None

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103


class FakeWizard(object):
    """Replace wizard() function on wizard pages."""

    customButtonClicked = QtCore.QObject()
    sign_in_page_id = 1
    setup_account_id = 2
    current_user_id = 3
    email_verification_id = 4
    success_id = 5
    error_id = 6
    forgotten_id = 7
    reset_password_id = 8

    def __init__(self):
        self.overlay = FakeOverlay()
        self.called = []
        self.buttons = {}
        self.buttons_text = {}
        self._next_id = -1
        self.sign_in_page = object()
        self.setup_account = object()
        self.current_user = object()
        self.email_verification = object()
        self.success = object()
        self.error = object()
        self.forgotten = object()
        self.reset_password = object()
        self._pages = {
            self.sign_in_page: self.sign_in_page_id,
            self.setup_account: self.setup_account_id,
            self.current_user: self.current_user_id,
            self.email_verification: self.email_verification_id,
            self.success: self.success_id,
            self.error: self.error_id,
            self.forgotten: self.forgotten_id,
            self.reset_password: self.reset_password_id,
        }

    # Invalid name "setButtonLayout", "setOption"
    # pylint: disable=C0103

    def setButtonText(self, key, value):
        """Fake setButtonText."""
        self.buttons_text[key] = value

    def setButtonLayout(self, *args, **kwargs):
        """Fake the functionality of setButtonLayout on QWizard class."""
        self.called.append(('setButtonLayout', (args, kwargs)))

    def setOption(self, *args, **kwargs):
        """Fake the functionality of setOption on QWizard class."""
        self.called.append(('setOption', (args, kwargs)))

    # pylint: enable=C0103

    def button(self, button_id):
        """Fake the functionality of button on QWizard class."""
        return self.buttons.setdefault(button_id, QtGui.QPushButton())

    def next(self):
        """Fake next."""
        self.called.append('next')

    def page(self, p_id):
        """Fake page method."""
        return self

    def set_titles(self, title):
        """Fake set_titles."""
        self.called.append(('set_titles', title))


class FakeWizardButtonStyle(FakeWizard):

    """Fake Wizard with button style implementation."""

    SIGN_IN_PAGE_ID = 1

    # pylint: disable=C0103
    def __init__(self):
        super(FakeWizardButtonStyle, self).__init__()
        self.data = {}
        self.customButtonClicked = self

    def setDefault(self, value):
        """Fake setDefault for button."""
        self.data['default'] = value

    def isDefault(self):
        """Fake isDefault."""
        return self.data['default']
    # pylint: enable=C0103

    def connect(self, func):
        """Fake customButtonClicked connect."""
        self.data['connect'] = func

    def disconnect(self, func):
        """Fake customButtonClicked disconnect."""
        self.data['disconnect'] = func

    def button(self, button_id):
        """Fake the functionality of button on QWizard class."""
        return self

    def style(self):
        """Fake button style."""
        return self

    def polish(self, button):
        """Fake polish."""
        self.data['polish'] = button

    def unpolish(self, button):
        """Fake unpolish."""
        self.data['unpolish'] = button

    def next(self):
        """Fake next for wizard."""
        self.data['next'] = True


class BaseTestCase(TestCase):
    """The base test case."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(BaseTestCase, self).setUp()
        self._called = False
        backend = FakedBackend()
        self.sso_login_backend = backend.sso_login
        self.patch(main, 'get_sso_client',
                   lambda *a: defer.succeed(backend))

    def _set_called(self, *args, **kwargs):
        """Store 'args' and 'kwargs' for test assertions."""
        self._called = (args, kwargs)


class BaseTestCaseUI(TestCase):
    """Base Test Case."""

    class_ui = None
    kwargs = {}

    @defer.inlineCallbacks
    def setUp(self):
        yield super(BaseTestCaseUI, self).setUp()
        # self.class_ui is not callable
        # pylint: disable=E1102, C0103, W0212
        self.ui = None
        self._called = False
        if self.class_ui is not None:
            self.ui = self.class_ui(**self.kwargs)

        if hasattr(self.ui, 'backend'):
            # clean backend calls
            self.ui.backend._called.clear()

        self.patch(loadingoverlay, 'LoadingOverlay', FakeOverlay)

    def _set_called(self, *args, **kwargs):
        """Store 'args' and 'kwargs' for test assertions."""
        self._called = (args, kwargs)

    def get_pixmap_data(self, pixmap):
        """Get the raw data of a QPixmap."""
        byte_array = QtCore.QByteArray()
        array_buffer = QtCore.QBuffer(byte_array)
        pixmap.save(array_buffer, "PNG")
        return byte_array

    # Invalid name "assertEqualPixmap"
    # pylint: disable=C0103

    def assertEqualPixmaps(self, pixmap1, pixmap2):
        """Compare two Qt pixmaps."""
        d1 = self.get_pixmap_data(pixmap1)
        d2 = self.get_pixmap_data(pixmap2)
        self.assertEqual(d1, d2)
