# -*- coding: utf-8 -*-
# Author: Diego Sarmentero <diego.sarmentero@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Test the common functions."""

from PyQt4 import QtGui
from twisted.internet import defer
from twisted.trial.unittest import TestCase

from ubuntu_sso.qt.common import (check_as_invalid,
    check_as_valid,
    password_assistance,
    password_check_match,
    BAD,
    GOOD,
    NORMAL,
    PASSWORD_DIGIT,
    PASSWORD_LENGTH,
    PASSWORD_MATCH,
    PASSWORD_UPPER)


class PasswordTestCase(TestCase):
    """Test passwords conditions."""

    @defer.inlineCallbacks
    def setUp(self):
        """Setup tests."""
        yield super(PasswordTestCase, self).setUp()
        self.line_edit = QtGui.QLineEdit()
        self.line_edit_confirm = QtGui.QLineEdit()
        self.label_assistance = QtGui.QLabel()

    def test_short_password(self):
        """Status with short password.

        * Password assistance contains the right message.
        """
        self.line_edit.setText("foobar")
        password_assistance(self.line_edit, self.label_assistance)
        self.assertIn(
            BAD % PASSWORD_LENGTH,
            unicode(self.label_assistance.text()),
        )

    def test_long_password(self):
        """Status with long password.

        * Password assistance contains the right message.
        """
        self.line_edit.setText("foobarbaz")
        password_assistance(self.line_edit, self.label_assistance)
        self.assertIn(
            GOOD % PASSWORD_LENGTH,
            unicode(self.label_assistance.text()),
        )

    def test_no_number_password(self):
        """Status with password without a number.

        * Password assistance contains the right message.
        """
        self.line_edit.setText("foobarbaz")
        password_assistance(self.line_edit, self.label_assistance)
        self.assertIn(
            BAD % PASSWORD_DIGIT,
            unicode(self.label_assistance.text()),
        )

    def test_number_password(self):
        """Status with password with a number.

        * Password assistance contains the right message.
        """
        self.line_edit.setText("foobarba7")
        password_assistance(self.line_edit, self.label_assistance)
        self.assertIn(
            GOOD % PASSWORD_DIGIT,
            unicode(self.label_assistance.text()),
        )

    def test_no_uppercase_password(self):
        """Status with password without uppercase letters.

        * Password assistance contains the right message.
        """
        self.line_edit.setText("foobarbaz")
        password_assistance(self.line_edit, self.label_assistance)
        self.assertIn(
            BAD % PASSWORD_UPPER,
            unicode(self.label_assistance.text()),
        )

    def test_upper_password(self):
        """Status with password with uppercase letters.

        * Password assistance contains the right message.
        """
        self.line_edit.setText("Foobarba7")
        password_assistance(self.line_edit, self.label_assistance)
        self.assertIn(
            GOOD % PASSWORD_UPPER,
            unicode(self.label_assistance.text()),
        )

    def test_matching_passwords(self):
        """Status when the passwords match.

        * Password assistance doesn't contain the message.
        """
        self.line_edit.setText("Foobarba7")
        self.line_edit_confirm.setText("Foobarba7")
        password_check_match(self.line_edit,
            self.line_edit_confirm, self.label_assistance)
        self.assertNotIn(
            BAD % PASSWORD_MATCH,
            unicode(self.label_assistance.text()))

    def test_not_matching_passwords(self):
        """Status when the passwords not match.

        * Password assistance contains the right message.
        """
        self.line_edit.setText("Foobarba7")
        self.line_edit_confirm.setText("BazBarFo0")
        password_check_match(self.line_edit,
            self.line_edit_confirm, self.label_assistance)
        self.assertIn(
            BAD % PASSWORD_MATCH,
            unicode(self.label_assistance.text()),
        )

    def test_password_assistance_in_focus_length_correct(self):
        """Check the QLineEdit for password when the length is correct."""
        self.line_edit.setText("aaaaaaaaa")
        self.line_edit_confirm.setText("")
        password_assistance(self.line_edit, self.label_assistance, NORMAL)
        self.assertIn(
            GOOD % PASSWORD_LENGTH,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            NORMAL % PASSWORD_UPPER,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            NORMAL % PASSWORD_DIGIT,
            unicode(self.label_assistance.text()),
        )

    def test_password_assistance_in_focus_with_upper(self):
        """Check the QLineEdit for password when it has an upper case char."""
        self.line_edit.setText("AAa")
        self.line_edit_confirm.setText("")
        password_assistance(self.line_edit, self.label_assistance, NORMAL)
        self.assertIn(
            NORMAL % PASSWORD_LENGTH,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            GOOD % PASSWORD_UPPER,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            NORMAL % PASSWORD_DIGIT,
            unicode(self.label_assistance.text()),
        )

    def test_password_assistance_in_focus_with_number(self):
        """Check the QLineEdit for password when it contains numbers."""
        self.line_edit.setText("a123")
        self.line_edit_confirm.setText("")
        password_assistance(self.line_edit, self.label_assistance, NORMAL)
        self.assertIn(
            NORMAL % PASSWORD_LENGTH,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            NORMAL % PASSWORD_UPPER,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            GOOD % PASSWORD_DIGIT,
            unicode(self.label_assistance.text()),
        )

    def test_password_assistance_in_focus_all_ok(self):
        """Check the QLineEdit for password when all the condition are ok."""
        self.line_edit.setText("T3st3rqw")
        self.line_edit_confirm.setText("T3st3rqw")
        password_assistance(self.line_edit, self.label_assistance, NORMAL)
        self.assertIn(
            GOOD % PASSWORD_LENGTH,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            GOOD % PASSWORD_UPPER,
            unicode(self.label_assistance.text()),
        )
        self.assertIn(
            GOOD % PASSWORD_DIGIT,
            unicode(self.label_assistance.text()),
        )
        self.assertNotIn(
            NORMAL % PASSWORD_MATCH,
            unicode(self.label_assistance.text()),
        )


class LineEditStyleTestCase(TestCase):
    """Test QLineEdit styles for errors."""

    def test_check_valid(self):
        """Check the propery value of a QLineEdit marked as valid."""
        line_edit = QtGui.QLineEdit()
        check_as_valid(line_edit)
        self.assertFalse(line_edit.property("formError").toBool())

    def test_check_invalid(self):
        """Check the propery value of a QLineEdit marked as invalid."""
        line_edit = QtGui.QLineEdit()
        check_as_invalid(line_edit)
        self.assertTrue(line_edit.property("formError").toBool())
