# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Setup Account page Qt UI."""

from PyQt4 import QtGui, QtCore
from twisted.internet import defer

from ubuntu_sso.qt import gui
from ubuntu_sso.qt import current_user_sign_in_page
from ubuntu_sso.qt.tests import (
    BaseTestCase,
    FakedBackend,
    FakedObject,
    FakePageUiStyle,
    FakeWizard,
    FakeWizardButtonStyle,
)
from ubuntu_sso.qt.ui import current_user_sign_in_ui


# pylint: disable=W0212
class CurrentUserSignInTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(CurrentUserSignInTestCase, self).setUp()
        self.patch(gui.main, "get_sso_client", FakedBackend)
        self.ui = current_user_sign_in_page.CurrentUserSignInPage(
            current_user_sign_in_ui.Ui_CurrentUserSignInPage(),
            parent=None)
        self.wizard = FakeWizard()
        self.patch(self.ui, 'wizard', lambda: self.wizard)

    def test_init(self):
        """Test the construction of the object."""
        self.assertTrue('LoggedIn' in self.ui._signals)
        self.assertTrue('LoginError' in self.ui._signals)
        self.assertTrue('UserNotValidated' in self.ui._signals)
        self.assertTrue(callable(self.ui._signals['LoggedIn']))
        self.assertTrue(callable(self.ui._signals['LoginError']))
        self.assertTrue(callable(self.ui._signals['UserNotValidated']))
        self.assertEqual(self.ui.backend, FakedBackend.sso_login)

    # pylint: disable=E1101
    def test_setup_page(self):
        """Test that the backend is received and the methods are called."""
        exposed_methods = [
            '_set_translated_strings',
            '_setup_signals',
            '_connect_ui']
        self.patch(FakedObject, "exposed_methods", exposed_methods)
        faked_object = FakedObject()
        self.patch(self.ui, "_set_translated_strings",
            faked_object._set_translated_strings)
        self.patch(self.ui, "_setup_signals",
            faked_object._setup_signals)
        self.patch(self.ui, "_connect_ui",
            faked_object._connect_ui)
        self.ui.setup_page()
        self.assertTrue('_set_translated_strings' in faked_object._called)
        self.assertTrue('_setup_signals' in faked_object._called)
        self.assertTrue('_connect_ui' in faked_object._called)
    # pylint: enable=E1101

    def test_on_user_not_validated(self):
        """Test the navigation flow on user not validated."""
        self.ui.on_user_not_validated()
        self.assertTrue('next' in self.wizard.called)
        self.assertEqual(self.ui.next, self.wizard.email_verification_id)

    def test_next_id(self):
        """Test if the object return the proper next id."""
        email = 'email@example.com'
        self.ui.ui.email_edit.setText(email)
        self.ui.on_user_not_validated()
        next_id = self.ui.nextId()
        self.assertEqual(next_id, self.wizard.email_verification_id)
        self.assertTrue(('set_titles', email) in self.wizard.called)

    def test_initialize_page(self):
        """Test the initialization method."""
        wizard = FakeWizardButtonStyle()
        button = FakePageUiStyle()
        self.patch(self.ui, "wizard", lambda: wizard)
        self.patch(self.ui.ui, "sign_in_button", button)
        self.patch(self.ui, "setButtonText", wizard.setButtonText)
        self.ui.initializePage()
        self.assertEqual(wizard.buttons_text[QtGui.QWizard.CancelButton],
            "Cancel")
        self.assertTrue(('setButtonLayout',
            (([QtGui.QWizard.BackButton, QtGui.QWizard.Stretch],), {})) in
            wizard.called)
        self.assertTrue(button.properties['default'])
        self.assertFalse(button.isEnabled())
        self.assertTrue(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_cleanup_page(self):
        """Test the cleanupPage method."""
        self.ui.cleanupPage()
        self.assertEqual(self.wizard._next_id, None)

    def test_set_translated_strings(self):
        """Test the translated string method."""
        self.ui._set_translated_strings()
        self.assertEqual(self.ui.ui.email_label.text(),
            current_user_sign_in_page.EMAIL_LABEL)
        self.assertEqual(self.ui.ui.password_label.text(),
            current_user_sign_in_page.LOGIN_PASSWORD_LABEL)
        self.assertEqual(self.ui.ui.forgot_password_label.text(),
            current_user_sign_in_page.FORGOTTEN_PASSWORD_BUTTON)
        self.assertEqual(self.ui.ui.sign_in_button.text(),
            current_user_sign_in_page.SIGN_IN_BUTTON)

    def test_connect_ui(self):
        """Test the connect ui method."""
        self.ui._connect_ui()
        # We expect 2 values because _connect_ui is called in the init too.
        self.assertEqual(self.ui.ui.forgot_password_label.receivers(
            QtCore.SIGNAL("linkActivated(const QString&)")), 2)
        self.assertEqual(self.ui.ui.email_edit.receivers(
            QtCore.SIGNAL("textChanged(const QString&)")), 2)
        self.assertEqual(self.ui.ui.password_edit.receivers(
            QtCore.SIGNAL("textChanged(const QString&)")), 2)
        self.assertEqual(self.ui.ui.sign_in_button.receivers(
            QtCore.SIGNAL("clicked()")), 2)

    def test_validate_not_valid(self):
        """Test validate method."""
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "sign_in_button", button)
        self.ui.ui.email_edit.setText('not-valid-email')
        self.ui.ui.password_edit.setText('123456')
        self.ui._validate()
        self.assertFalse(button.isEnabled())
        self.assertTrue(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_validate_valid(self):
        """Test validate method."""
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "sign_in_button", button)
        self.ui.ui.email_edit.setText('valid@email')
        self.ui.ui.password_edit.setText('123456')
        self.ui._validate()
        self.assertTrue(button.isEnabled())
        self.assertFalse(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_login(self):
        """Test the login method."""
        exposed_methods = ['login']
        self.patch(FakedObject, "exposed_methods", exposed_methods)
        faked_object = FakedObject()
        self.patch(self.ui, "backend", faked_object)
        self.ui.ui.email_edit.setText('valid@email')
        self.ui.ui.password_edit.setText('123456')
        self.ui.login()
        self.assertTrue('login' in self.ui.backend._called)
        self.assertEqual(self.ui.backend._called['login'],
            ((None, u'valid@email', u'123456'), {}))

    def test_on_login_error(self):
        """Test the on_login_error method."""
        self.patch(self.ui.message_box, "critical", self._set_called)
        app_name = 'my_app'
        self.ui.app_name = app_name
        error = {'errtype': 'UserNotValidated'}
        self.ui.on_login_error(app_name, error)
        expected = ((self.ui, 'my_app', ''), {})
        self.assertTrue(expected, self._called)

    def test_on_logged_in(self):
        """Test the on_login_error method."""
        app_name = 'my_app'
        self.ui.app_name = app_name
        self.ui.on_logged_in(app_name, None)
        self.assertEqual(self.ui.next, self.wizard.success_id)

    def test_on_forgotten_password(self):
        """Test the on_login_error method."""
        self.ui.on_forgotten_password()
        self.assertEqual(self.ui.next, self.wizard.forgotten_id)
