# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Setup Account page Qt UI."""

from PyQt4 import QtGui, QtCore
from twisted.internet import defer

from ubuntu_sso.qt import gui
from ubuntu_sso.qt import email_verification_page
from ubuntu_sso.qt.tests import (
    BaseTestCase,
    FakedBackend,
    FakedObject,
    FakePageUiStyle,
    FakeWizard,
)
from ubuntu_sso.qt.ui import email_verification_ui


# pylint: disable=W0212
class EmailVerificationTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(EmailVerificationTestCase, self).setUp()
        self.patch(gui.main, "get_sso_client", FakedBackend)
        self.ui = email_verification_page.EmailVerificationPage(
            email_verification_ui.Ui_EmailVerificationPage(),
            parent=None)
        self.wizard = FakeWizard()
        self.patch(self.ui, 'wizard', lambda: self.wizard)

    def test_init(self):
        """Test the construction of the object."""
        self.assertTrue('EmailValidated' in self.ui._signals)
        self.assertTrue('EmailValidationError' in self.ui._signals)
        self.assertTrue(callable(self.ui._signals['EmailValidated']))
        self.assertTrue(callable(self.ui._signals['EmailValidationError']))
        self.assertEqual(self.ui.backend, FakedBackend.sso_login)

    # pylint: disable=E1101
    def test_setup_page(self):
        """Test that the backend is received and the methods are called."""
        exposed_methods = [
            '_setup_signals',
            '_connect_ui_elements']
        self.patch(FakedObject, "exposed_methods", exposed_methods)
        faked_object = FakedObject()
        self.patch(self.ui, "_setup_signals",
            faked_object._setup_signals)
        self.patch(self.ui, "_connect_ui_elements",
            faked_object._connect_ui_elements)
        self.ui.setup_page()
        self.assertTrue('_setup_signals' in faked_object._called)
        self.assertTrue('_connect_ui_elements' in faked_object._called)
    # pylint: enable=E1101

    def test_verification_code(self):
        """Test the verification value returned."""
        self.ui.ui.verification_code_edit.setText('asd123')
        value = self.ui.verification_code
        self.assertEqual(value, 'asd123')
        self.assertTrue(isinstance(value, str))

    def test_next_button(self):
        """Test the next button."""
        value = self.ui.next_button
        self.assertEqual(value, self.ui.ui.next_button)
        self.assertTrue(isinstance(value, QtGui.QPushButton))

    def test_connect_ui_elements(self):
        """Test the connect ui method."""
        self.ui._connect_ui_elements()
        # We expect 2 values because _connect_ui is called in the init too.
        self.assertEqual(self.ui.ui.verification_code_edit.receivers(
            QtCore.SIGNAL("textChanged(const QString&)")), 2)
        self.assertEqual(self.ui.next_button.receivers(
            QtCore.SIGNAL("clicked()")), 2)

    def test_validate_form_not_valid(self):
        """Test validate method."""
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "next_button", button)
        self.ui.ui.verification_code_edit.setText('  ')
        self.ui.validate_form()
        self.assertFalse(button.isEnabled())
        self.assertTrue(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_validate_form_valid(self):
        """Test validate method."""
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "next_button", button)
        self.ui.ui.verification_code_edit.setText('asd123')
        self.ui.validate_form()
        self.assertTrue(button.isEnabled())
        self.assertFalse(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_set_titles(self):
        """Test the set_titles method."""
        email = 'mail@example'
        app_name = 'my_app'
        self.ui.app_name = app_name
        self.ui.email = email
        self.ui.set_titles(email)
        self.assertEqual(self.ui.header.title_label.text(),
            email_verification_page.VERIFY_EMAIL_TITLE)
        expected = email_verification_page.VERIFY_EMAIL_CONTENT % {
                       "app_name": app_name,
                       "email": email,
                   }
        self.assertEqual(str(self.ui.header.subtitle_label.text()),
            expected)

    def test_initialize_page(self):
        """Test the initialization method."""
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "next_button", button)
        self.ui.initializePage()
        self.assertTrue(button.properties['default'])
        self.assertFalse(button.isEnabled())
        self.assertTrue(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_on_email_validation_error(self):
        """Test the validate_email method."""
        self.patch(self.ui.message_box, "critical", self._set_called)
        app_name = 'my_app'
        error = {email_verification_page: 'error in email_verification_page'}
        self.ui.on_email_validation_error(app_name, error)
        expected = ((self.ui, app_name, ''), {})
        self.assertTrue(expected, self._called)
