# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Setup Account page Qt UI."""

from PyQt4 import QtGui, QtCore
from twisted.internet import defer

from ubuntu_sso.qt import gui
from ubuntu_sso.qt import forgotten_password_page
from ubuntu_sso.qt.tests import (
    BaseTestCase,
    FakedBackend,
    FakedObject,
    FakePageUiStyle,
    FakeWizard,
)
from ubuntu_sso.qt.ui import forgotten_password_ui


# pylint: disable=W0212, E1101
class ForgottenPasswordTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(ForgottenPasswordTestCase, self).setUp()
        self.patch(gui.main, "get_sso_client", FakedBackend)
        self.ui = forgotten_password_page.ForgottenPasswordPage(
            forgotten_password_ui.Ui_ForgottenPasswordPage(),
            parent=None)
        self.wizard = FakeWizard()
        self.patch(self.ui, 'wizard', lambda: self.wizard)

    def test_init(self):
        """Test the construction of the object."""
        self.assertTrue('PasswordResetTokenSent' in self.ui._signals)
        self.assertTrue('PasswordResetError' in self.ui._signals)
        self.assertTrue(callable(self.ui._signals['PasswordResetTokenSent']))
        self.assertTrue(callable(self.ui._signals['PasswordResetError']))
        self.assertEqual(self.ui.backend, FakedBackend.sso_login)

    def test_setup_page(self):
        """Test that the backend is received and the methods are called."""
        exposed_methods = [
            '_setup_signals',
            '_set_translated_strings',
            '_connect_ui',
            '_set_enhanced_line_edit',
            '_register_fields']
        self.patch(FakedObject, "exposed_methods", exposed_methods)
        faked_object = FakedObject()
        self.patch(self.ui, "_setup_signals",
            faked_object._setup_signals)
        self.patch(self.ui, "_set_translated_strings",
            faked_object._set_translated_strings)
        self.patch(self.ui, "_connect_ui",
            faked_object._connect_ui)
        self.patch(self.ui, "_set_enhanced_line_edit",
            faked_object._set_enhanced_line_edit)
        self.patch(self.ui, "_register_fields",
            faked_object._register_fields)
        self.ui.setup_page()
        self.assertTrue('_setup_signals' in faked_object._called)
        self.assertTrue('_set_translated_strings' in faked_object._called)
        self.assertTrue('_connect_ui' in faked_object._called)
        self.assertTrue('_set_enhanced_line_edit' in faked_object._called)
        self.assertTrue('_register_fields' in faked_object._called)

    def test_email_widget(self):
        """Test the email_widget property."""
        value = self.ui.email_widget
        self.assertEqual(value, self.ui.ui.email_widget)
        self.assertTrue(isinstance(value, QtGui.QWidget))

    def test_forgotted_password_intro_label(self):
        """Test the forgotted_password_intro_label property."""
        value = self.ui.forgotted_password_intro_label
        self.assertEqual(value, self.ui.ui.forgotted_password_intro_label)
        self.assertTrue(isinstance(value, QtGui.QLabel))

    def test_email_address_label(self):
        """Test the email_address_label property."""
        value = self.ui.email_address_label
        self.assertEqual(value, self.ui.ui.email_address_label)
        self.assertTrue(isinstance(value, QtGui.QLabel))

    def test_email_address(self):
        """Test the email_address property."""
        value = self.ui.email_address
        self.assertEqual(value, str(self.ui.ui.email_line_edit.text()))
        self.assertTrue(isinstance(value, str))

    def test_email_address_line_edit(self):
        """Test the email_address_line_edit property."""
        value = self.ui.email_address_line_edit
        self.assertEqual(value, self.ui.ui.email_line_edit)
        self.assertTrue(isinstance(value, QtGui.QLineEdit))

    def test_send_button(self):
        """Test the send_button property."""
        value = self.ui.send_button
        self.assertEqual(value, self.ui.ui.send_button)
        self.assertTrue(isinstance(value, QtGui.QPushButton))

    def test_try_again_widget(self):
        """Test the try_again_widget property."""
        value = self.ui.try_again_widget
        self.assertEqual(value, self.ui.ui.try_again_widget)
        self.assertTrue(isinstance(value, QtGui.QWidget))

    def test_try_again_button(self):
        """Test the try_again_button property."""
        value = self.ui.try_again_button
        self.assertEqual(value, self.ui.ui.try_again_button)
        self.assertTrue(isinstance(value, QtGui.QPushButton))

    def test_initialize_page_with_email(self):
        """Test the initialization method."""
        self.ui.ui.email_line_edit.setText('mail@example')
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "send_button", button)
        self.ui.initializePage()
        self.assertTrue(button.properties['default'])
        self.assertTrue(button.isEnabled())
        self.assertFalse(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_set_translated_strings(self):
        """Test the translated string method."""
        app_name = 'my_app'
        self.ui.app_name = app_name
        self.ui._set_translated_strings()
        expected = (forgotten_password_page.REQUEST_PASSWORD_TOKEN_LABEL %
            {'app_name': app_name})
        self.assertEqual(self.ui.ui.forgotted_password_intro_label.text(),
            expected)
        self.assertEqual(self.ui.ui.email_address_label.text(),
            forgotten_password_page.EMAIL_LABEL)
        self.assertEqual(self.ui.ui.send_button.text(),
            forgotten_password_page.RESET_PASSWORD)
        self.assertEqual(self.ui.ui.try_again_button.text(),
            forgotten_password_page.TRY_AGAIN_BUTTON)

    def test_connect_ui(self):
        """Test the connect ui method."""
        self.ui._connect_ui()
        # We expect 2 values because _connect_ui is called in the init too.
        # For email_line_edit we expect 3 because has another
        # connection in the .ui file.
        self.assertEqual(self.ui.ui.email_line_edit.receivers(
            QtCore.SIGNAL("textChanged(const QString&)")), 3)
        self.assertEqual(self.ui.ui.send_button.receivers(
            QtCore.SIGNAL("clicked()")), 2)
        self.assertEqual(self.ui.ui.try_again_button.receivers(
            QtCore.SIGNAL("clicked()")), 2)

    def test_validate_not_valid(self):
        """Test validate method."""
        self.ui.email_address_line_edit.setText('')
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "send_button", button)
        self.ui._validate()
        self.assertFalse(button.isEnabled())
        self.assertTrue(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_validate_valid(self):
        """Test validate method."""
        self.ui.email_address_line_edit.setText('mail@example')
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "send_button", button)
        self.ui._validate()
        self.assertTrue(button.isEnabled())
        self.assertFalse(button.property("DisabledState"))
        self.assertTrue(button.properties['unpolish'])
        self.assertTrue(button.properties['polish'])

    def test_on_try_again(self):
        """Test on_try_again method."""
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.on_try_again()
        self.assertTrue(self.ui.email_widget.isVisible())
        self.assertFalse(self.ui.try_again_widget.isVisible())

    def test_on_password_reset_token_sent(self):
        """Test on_password_reset_token_sent method."""
        self.ui.on_password_reset_token_sent()
        self.assertEqual(self.ui.next, self.wizard.reset_password_id)

    def test_on_password_reset_error(self):
        """Test on_password_reset_error method."""
        self.patch(self.ui.message_box, "critical", self._set_called)
        app_name = 'my_app'
        error = {'errtype': 'AnotherError'}
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.on_password_reset_error(app_name, error)
        expected = ((self.ui, app_name,
            forgotten_password_page.REQUEST_PASSWORD_TOKEN_TECH_ERROR), {})
        self.assertTrue(expected, self._called)
        self.assertFalse(self.ui.email_widget.isVisible())
        self.assertFalse(self.ui.forgotted_password_intro_label.isVisible())
        self.assertTrue(self.ui.try_again_widget.isVisible())

    def test_on_password_reset_error_with_token_error(self):
        """Test on_password_reset_error method."""
        self.patch(self.ui.message_box, "critical", self._set_called)
        app_name = 'my_app'
        error = {'errtype': 'ResetPasswordTokenError'}
        self.ui.on_password_reset_error(app_name, error)
        expected = ((self.ui, app_name,
            forgotten_password_page.REQUEST_PASSWORD_TOKEN_WRONG_EMAIL), {})
        self.assertTrue(expected, self._called)
