# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Test the Reset Password Page."""

# pylint: disable=F0401,E0611,E1101

from PyQt4 import QtGui, QtCore
from twisted.internet import defer
from twisted.trial.unittest import TestCase

from ubuntu_sso.utils.ui import (
    PASSWORD1_ENTRY,
    PASSWORD2_ENTRY,
    RESET_CODE_ENTRY)
from ubuntu_sso.qt import gui
from ubuntu_sso.qt import common
from ubuntu_sso.qt.reset_password_page import (
    ResetPasswordPage,
    RESET_SUBTITLE,
    RESET_TITLE,
)
from ubuntu_sso.qt.ui.reset_password_ui import Ui_ResetPasswordPage
from ubuntu_sso.qt.tests import FakedBackend


class FakePasswordAssistance(object):
    """Fake password_assistance_* calls."""

    _called = False

    def call(self, *args, **kwargs):
        """Check if the method was called."""
        FakePasswordAssistance._called = True

    def clenaup(self):
        """Clean up the variable."""
        FakePasswordAssistance._called = False


class ResetPasswordTestCase(TestCase):
    """Test the ResetPasswordPage code."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(ResetPasswordTestCase, self).setUp()
        self.patch(gui.main, "get_sso_client", FakedBackend)
        self.page = ResetPasswordPage(Ui_ResetPasswordPage(),
                                                'app_name',
                                                None)
        self.fake = FakePasswordAssistance()

    def test_init(self):
        """Check the initial state of ResetPassword."""
        self.assertEqual(self.page.ui.password_line_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)
        self.assertEqual(self.page.ui.confirm_password_line_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)

    def test_initialize(self):
        """Check the Title and Subtitle."""
        self.page.show()
        self.page.initializePage()
        self.addCleanup(self.page.hide)
        self.assertEqual(self.page.header.title_label.text(), RESET_TITLE)
        self.assertEqual(self.page.header.subtitle_label.text(),
            RESET_SUBTITLE)
        self.assertEqual(self.page.ui.password_label.text(), PASSWORD1_ENTRY)
        self.assertEqual(self.page.ui.confirm_password_label.text(),
            PASSWORD2_ENTRY)
        self.assertEqual(self.page.ui.reset_code.text(), RESET_CODE_ENTRY)

    def test_focus_changed_password_visibility(self):
        """Check visibility changes when focus_changed() is executed."""
        self.page.show()
        self.addCleanup(self.page.hide)
        self.page.focus_changed(None, self.page.ui.password_line_edit)
        self.assertTrue(self.page.ui.password_assistance.isVisible())

    def test_show_hide_event(self):
        """Check connections to focusChanged on show and hide event."""
        self.page.show()
        self.addCleanup(self.page.hide)
        self.assertEqual(QtGui.QApplication.instance().receivers(
            QtCore.SIGNAL("focusChanged(QWidget*, QWidget*)")), 1)
        self.page.hide()
        self.assertEqual(QtGui.QApplication.instance().receivers(
            QtCore.SIGNAL("focusChanged(QWidget*, QWidget*)")), 0)

    # pylint: disable=W0212

    def test_focus_changed_1(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(common, 'password_default_assistance', self.fake.call)
        self.addCleanup(self.fake.clenaup)
        self.page.show()
        self.addCleanup(self.page.hide)
        self.assertFalse(FakePasswordAssistance._called)
        self.page.focus_changed(None, self.page.ui.password_line_edit)
        self.assertTrue(self.page.ui.password_assistance.isVisible())
        self.assertTrue(FakePasswordAssistance._called)

    def test_focus_changed_2(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(common, 'password_check_match', self.fake.call)
        self.addCleanup(self.fake.clenaup)
        self.page.show()
        self.addCleanup(self.page.hide)
        self.assertFalse(FakePasswordAssistance._called)
        self.page.focus_changed(None, self.page.ui.confirm_password_line_edit)
        self.assertTrue(self.page.ui.password_assistance.isVisible())
        self.assertTrue(FakePasswordAssistance._called)

    # pylint: enable=W0212
