# -*- coding: utf-8 -*-

# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""SSO Wizard UI."""

from PyQt4.QtCore import pyqtSignal
from PyQt4.QtGui import (
    QPixmap,
    QWizard,
)
from twisted.internet import defer

from ubuntu_sso import NO_OP
from ubuntu_sso.qt.current_user_sign_in_page import CurrentUserSignInPage
from ubuntu_sso.qt.email_verification_page import EmailVerificationPage
from ubuntu_sso.qt.error_page import ErrorPage
from ubuntu_sso.qt.forgotten_password_page import ForgottenPasswordPage
from ubuntu_sso.qt.reset_password_page import ResetPasswordPage
from ubuntu_sso.qt.setup_account_page import SetupAccountPage
from ubuntu_sso.qt.sign_in_page import SignInPage
from ubuntu_sso.qt.success_page import SuccessPage
from ubuntu_sso.qt.ui.choose_sign_in_ui import Ui_ChooseSignInPage
from ubuntu_sso.qt.ui.current_user_sign_in_ui import Ui_CurrentUserSignInPage
from ubuntu_sso.qt.ui.email_verification_ui import Ui_EmailVerificationPage
from ubuntu_sso.qt.ui.error_message_ui import Ui_ErrorPage
from ubuntu_sso.qt.ui.setup_account_ui import Ui_SetUpAccountPage
from ubuntu_sso.qt.ui.success_message_ui import Ui_SuccessPage
from ubuntu_sso.qt.ui.forgotten_password_ui import Ui_ForgottenPasswordPage
from ubuntu_sso.qt.ui.reset_password_ui import Ui_ResetPasswordPage
from ubuntu_sso.logger import setup_logging


logger = setup_logging('ubuntu_sso.gui')


class UbuntuSSOWizard(QWizard):
    """Wizard used to create or use sso."""

    # definition of the signals raised by the widget
    recoverableError = pyqtSignal('QString', 'QString')
    loginSuccess = pyqtSignal('QString', 'QString')
    registrationSuccess = pyqtSignal('QString', 'QString')

    def __init__(self, app_name, **kwargs):
        """Create a new wizard."""
        parent = kwargs.get('parent')
        super(UbuntuSSOWizard, self).__init__(parent)

        # store common useful data provided by the app
        self.app_name = app_name
        self.ping_url = kwargs.get('ping_url', '')
        self.tc_url = kwargs.get('tc_url', '')
        self.policy_url = kwargs.get('policy_url', '')
        self.help_text = kwargs.get('help_text', '')
        self.login_only = kwargs.get('login_only', False)
        self.close_callback = kwargs.get('close_callback', lambda: None)
        self.login_success_callback = NO_OP
        self.registration_success_callback = NO_OP
        self.user_cancellation_callback = NO_OP

        # set the diff pages of the QWizard
        self.sign_in_page = SignInPage(Ui_ChooseSignInPage(),
                                       QPixmap(),
                                       app_name=self.app_name,
                                       title='Sign in',
                                       parent=self)
        self.setup_account = SetupAccountPage(Ui_SetUpAccountPage(),
                                              self.help_text,
                                              self.tc_url,
                                              self.policy_url,
                                              app_name=self.app_name,
                                              parent=self)
        self.current_user = CurrentUserSignInPage(Ui_CurrentUserSignInPage(),
                                                  app_name=self.app_name,
                                                  parent=self)
        self.email_verification = EmailVerificationPage(
                                                Ui_EmailVerificationPage(),
                                                app_name=self.app_name)
        self.success = SuccessPage(Ui_SuccessPage(),
                                   app_name=self.app_name, parent=self)
        self.error = ErrorPage(Ui_ErrorPage(),
                               app_name=self.app_name)
        self.forgotten = ForgottenPasswordPage(Ui_ForgottenPasswordPage(),
                                               app_name=self.app_name,
                                               parent=self)
        self.reset_password = ResetPasswordPage(Ui_ResetPasswordPage(),
                                                app_name=self.app_name,
                                                parent=self)
        # store the ids of the pages so that it is easier to access them later
        pages = [self.sign_in_page, self.setup_account,
                     self.email_verification, self.current_user, self.success,
                     self.error, self.forgotten, self.reset_password]
        self._pages = {}
        for page in pages:
            self._pages[page] = self.addPage(page)

        # set the buttons layout to only have cancel and back since the next
        # buttons are the ones used in the diff pages.
        buttons_layout = []
        buttons_layout.append(QWizard.Stretch)
        buttons_layout.append(QWizard.BackButton)
        buttons_layout.append(QWizard.CancelButton)
        self.setButtonLayout(buttons_layout)
        self.setWindowTitle(self.app_name)
        self.setWizardStyle(QWizard.ModernStyle)
        self.button(QWizard.CancelButton).clicked.connect(
                                                    self.on_user_cancelation)
        self.loginSuccess.connect(self.on_login_success)
        self.registrationSuccess.connect(self.on_registration_success)

    @property
    def sign_in_page_id(self):
        """Return the id of the page used for choosing sign in type."""
        return self._pages[self.sign_in_page]

    @property
    def setup_account_page_id(self):
        """Return the id of the page used for sign in."""
        return self._pages[self.setup_account]

    @property
    def email_verification_page_id(self):
        """Return the id of the verification page."""
        return self._pages[self.email_verification]

    @property
    def current_user_page_id(self):
        """Return the id used to signin by a current user."""
        return self._pages[self.current_user]

    @property
    def success_page_id(self):
        """Return the id of the success page."""
        return self._pages[self.success]

    @property
    def forgotten_password_page_id(self):
        """Return the id of the forgotten password page."""
        return self._pages[self.forgotten]

    @property
    def reset_password_page_id(self):
        """Return the id of the reset password page."""
        return self._pages[self.reset_password]

    @property
    def error_page_id(self):
        """Return the id of the error page."""
        return self._pages[self.error]

    def on_user_cancelation(self):
        """Process the cancel action."""
        logger.debug('UbuntuSSOWizard.on_user_cancelation')
        self.user_cancellation_callback(self.app_name)
        self.close()

    @defer.inlineCallbacks
    def on_login_success(self, app_name, email):
        """Process the success of a login."""
        logger.debug('UbuntuSSOWizard.on_login_success')
        result = yield self.login_success_callback(
            unicode(app_name), unicode(email))
        logger.debug('Result from callback is %s', result)
        if result == 0:
            logger.info('Success in calling the given success_callback')
            self.show_success_message()
        else:
            logger.info('Error in calling the given success_callback')
            self.show_error_message()

    @defer.inlineCallbacks
    def on_registration_success(self, app_name, email):
        """Process the success of a registration."""
        logger.debug('UbuntuSSOWizard.on_registration_success')
        result = yield self.registration_success_callback(unicode(app_name),
                                                          unicode(email))
        logger.debug('Result from callback is %s', result)
        if result == 0:
            logger.info('Success in calling the given registration_callback')
            self.show_success_message()
        else:
            logger.info('Success in calling the given registration_callback')
            self.show_error_message()

    def show_success_message(self):
        """Show the success message in the view."""
        logger.info('Showing success message.')
        # get the id of the success page, set it as the next id of the
        # current page and let the wizard move to the next step
        self.currentPage().next = self.success_page_id
        self.next()
        # show the finish button but with a close message
        buttons_layout = []
        buttons_layout.append(QWizard.Stretch)
        buttons_layout.append(QWizard.FinishButton)
        self.setButtonLayout(buttons_layout)

    def show_error_message(self):
        """Show the error page in the view."""
        logger.info('Showing error message.')
        # similar to the success page but using the error id
        self.currentPage().next = self.error_page_id
        self.next()
        # show the finish button but with a close message
        buttons_layout = []
        buttons_layout.append(QWizard.Stretch)
        buttons_layout.append(QWizard.FinishButton)
        self.setButtonLayout(buttons_layout)


class UbuntuSSOClientGUI(object):
    """Ubuntu single sign-on GUI."""

    def __init__(self, app_name, **kwargs):
        """Create a new instance."""
        super(UbuntuSSOClientGUI, self).__init__()
        logger.debug('UbuntuSSOClientGUI: app_name %r, kwargs %r.',
                     app_name, kwargs)
        self.app_name = app_name
        # create the controller and the ui, then set the cb and call the show
        # method so that we can work
        self.view = UbuntuSSOWizard(app_name=app_name, **kwargs)
        self.view.show()

    def get_login_success_callback(self):
        """Return the log in success cb."""
        return self.view.login_success_callback

    def set_login_success_callback(self, cb):
        """Set log in success cb."""
        self.view.login_success_callback = cb

    login_success_callback = property(get_login_success_callback,
                                      set_login_success_callback)

    def get_registration_success_callback(self):
        """Return the registration success cb."""
        return self.view.registration_success_callback

    def set_registration_success_callback(self, cb):
        """Set registration success cb."""
        self.view.registration_success_callback = cb

    registration_success_callback = property(get_registration_success_callback,
                                             set_registration_success_callback)

    def get_user_cancellation_callback(self):
        """Return the user cancellation callback."""
        return self.view.user_cancellation_callback

    def set_user_cancellation_callback(self, cb):
        """Set the user cancellation callback."""
        self.view.user_cancellation_callback = cb

    user_cancellation_callback = property(get_user_cancellation_callback,
                                          set_user_cancellation_callback)
