# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""The common bits of a webclient."""

import cgi
import collections

from httplib2 import iri2uri
from oauth import oauth
from twisted.internet import defer
from urlparse import urlparse

from ubuntu_sso.utils.webclient.timestamp import TimestampChecker


class WebClientError(Exception):
    """An http error happened while calling the webservice."""


class UnauthorizedError(WebClientError):
    """The request ended with bad_request, unauthorized or forbidden."""


class Response(object):
    """A reponse object."""

    def __init__(self, content, headers=None):
        """Initialize this instance."""
        self.content = content
        self.headers = headers


class HeaderDict(collections.defaultdict):
    """A case insensitive dict for headers."""

    # pylint: disable=E1002
    def __init__(self, *args, **kwargs):
        """Handle case-insensitive keys."""
        super(HeaderDict, self).__init__(list, *args, **kwargs)
        # pylint: disable=E1101
        for key, value in self.items():
            super(HeaderDict, self).__delitem__(key)
            self[key] = value

    def __setitem__(self, key, value):
        """Set the value with a case-insensitive key."""
        super(HeaderDict, self).__setitem__(key.lower(), value)

    def __getitem__(self, key):
        """Get the value with a case-insensitive key."""
        return super(HeaderDict, self).__getitem__(key.lower())

    def __delitem__(self, key):
        """Delete the item with the case-insensitive key."""
        super(HeaderDict, self).__delitem__(key.lower())

    def __contains__(self, key):
        """Check the containment with a case-insensitive key."""
        return super(HeaderDict, self).__contains__(key.lower())


class BaseWebClient(object):
    """The webclient base class, to be extended by backends."""

    timestamp_checker = None

    def __init__(self, username=None, password=None, oauth_sign_plain=False):
        """Initialize this instance."""
        self.username = username
        self.password = password
        self.oauth_sign_plain = oauth_sign_plain

    def request(self, iri, method="GET", extra_headers=None,
                oauth_credentials=None, post_content=None):
        """Return a deferred that will be fired with a Response object."""
        raise NotImplementedError

    @classmethod
    def get_timestamp_checker(cls):
        """Get the timestamp checker for this class of webclient."""
        if cls.timestamp_checker is None:
            cls.timestamp_checker = TimestampChecker(cls)
        return cls.timestamp_checker

    def get_timestamp(self):
        """Get a timestamp synchronized with the server."""
        return self.get_timestamp_checker().get_faithful_time()

    def force_use_proxy(self, settings):
        """Setup this webclient to use the given proxy settings."""
        raise NotImplementedError

    def iri_to_uri(self, iri):
        """Transform a unicode iri into a ascii uri."""
        if not isinstance(iri, unicode):
            raise TypeError('iri %r should be unicode.' % iri)
        return bytes(iri2uri(iri))

    def build_oauth_request(self, method, uri, credentials, timestamp,
                            parameters=None):
        """Build an oauth request given some credentials."""
        consumer = oauth.OAuthConsumer(credentials["consumer_key"],
                                       credentials["consumer_secret"])
        token = oauth.OAuthToken(credentials["token"],
                                 credentials["token_secret"])
        if not parameters:
            _, _, _, _, query, _ = urlparse(uri)
            parameters = dict(cgi.parse_qsl(query))

        if timestamp:
            parameters["oauth_timestamp"] = timestamp

        request = oauth.OAuthRequest.from_consumer_and_token(
                                            http_url=uri,
                                            http_method=method,
                                            parameters=parameters,
                                            oauth_consumer=consumer,
                                            token=token)
        if self.oauth_sign_plain:
            sig_method = oauth.OAuthSignatureMethod_PLAINTEXT()
        else:
            sig_method = oauth.OAuthSignatureMethod_HMAC_SHA1()
        request.sign_request(sig_method, consumer, token)
        return request

    def build_oauth_headers(self, method, uri, credentials, timestamp):
        """Build oauth request headers given some credentials."""
        request = self.build_oauth_request(method, uri, credentials, timestamp)
        return request.to_header()

    @defer.inlineCallbacks
    def build_signed_iri(self, iri, credentials, parameters=None):
        """Build a new iri signing 'iri' with 'credentials'."""
        uri = self.iri_to_uri(iri)
        timestamp = yield self.get_timestamp()
        request = self.build_oauth_request(method='GET', uri=uri,
                                           credentials=credentials,
                                           timestamp=timestamp,
                                           parameters=parameters)
        defer.returnValue(request.to_url())

    def shutdown(self):
        """Shut down all pending requests (if possible)."""
