# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT AN WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Integration tests for the proxy-enabled webclient."""

import os
import sys
import urllib

from twisted.cred import checkers, portal
from twisted.internet import defer
from twisted.web import guard, http, resource
from urllib2 import urlparse

from ubuntuone.devtools.testcases import TestCase
from ubuntuone.devtools.testcases.squid import SquidTestCase

from ubuntu_sso.utils import webclient
from ubuntu_sso.utils.webclient.common import BaseWebClient, HeaderDict, oauth
from ubuntu_sso.utils.webclient.tests import BaseMockWebServer

ANY_VALUE = object()
SAMPLE_KEY = "result"
SAMPLE_VALUE = "sample result"
SAMPLE_RESOURCE = '{"%s": "%s"}' % (SAMPLE_KEY, SAMPLE_VALUE)
SAMPLE_USERNAME = "peddro"
SAMPLE_PASSWORD = "cantropus"
SAMPLE_CREDENTIALS = dict(
    consumer_key="consumer key",
    consumer_secret="consumer secret",
    token="the real token",
    token_secret="the token secret",
)
SAMPLE_HEADERS = {SAMPLE_KEY: SAMPLE_VALUE}
SAMPLE_POST_PARAMS = {"param1": "value1", "param2": "value2"}
SAMPLE_JPEG_HEADER = '\xff\xd8\xff\xe0\x00\x10JFIF'

SIMPLERESOURCE = "simpleresource"
BYTEZERORESOURCE = "bytezeroresource"
POSTABLERESOURCE = "postableresource"
THROWERROR = "throwerror"
UNAUTHORIZED = "unauthorized"
HEADONLY = "headonly"
VERIFYHEADERS = "verifyheaders"
VERIFYPOSTPARAMS = "verifypostparams"
GUARDED = "guarded"
OAUTHRESOURCE = "oauthresource"

WEBCLIENT_MODULE_NAME = webclient.webclient_module().__name__


def sample_get_credentials():
    """Will return the sample credentials right now."""
    return defer.succeed(SAMPLE_CREDENTIALS)


# pylint: disable=C0103
# t.w.resource methods have freeform cased names

class SimpleResource(resource.Resource):
    """A simple web resource."""

    def render_GET(self, request):
        """Make a bit of html out of these resource's content."""
        return SAMPLE_RESOURCE


class ByteZeroResource(resource.Resource):
    """A resource that has a nul byte in the middle of it."""

    def render_GET(self, request):
        """Return the content of this resource."""
        return SAMPLE_JPEG_HEADER


class PostableResource(resource.Resource):
    """A resource that only answers to POST requests."""

    def render_POST(self, request):
        """Make a bit of html out of these resource's content."""
        return SAMPLE_RESOURCE


class HeadOnlyResource(resource.Resource):
    """A resource that fails if called with a method other than HEAD."""

    def render_HEAD(self, request):
        """Return a bit of html."""
        return "OK"


class VerifyHeadersResource(resource.Resource):
    """A resource that verifies the headers received."""

    def render_GET(self, request):
        """Make a bit of html out of these resource's content."""
        headers = request.requestHeaders.getRawHeaders(SAMPLE_KEY)
        if headers != [SAMPLE_VALUE]:
            request.setResponseCode(http.BAD_REQUEST)
            return "ERROR: Expected header not present."
        request.setHeader(SAMPLE_KEY, SAMPLE_VALUE)
        return SAMPLE_RESOURCE


class VerifyPostParameters(resource.Resource):
    """A resource that answers to POST requests with some parameters."""

    def fetch_post_args_only(self, request):
        """Fetch only the POST arguments, not the args in the url."""
        request.process = lambda: None
        request.requestReceived(request.method, request.path,
                                request.clientproto)
        return request.args

    def render_POST(self, request):
        """Verify the parameters that we've been called with."""
        post_params = self.fetch_post_args_only(request)
        expected = dict((key, [val]) for key, val
                                      in SAMPLE_POST_PARAMS.items())
        if post_params != expected:
            request.setResponseCode(http.BAD_REQUEST)
            return "ERROR: Expected arguments not present, %r != %r" % (
                    post_params, expected)
        return SAMPLE_RESOURCE


class SimpleRealm(object):
    """The same simple resource for all users."""

    def requestAvatar(self, avatarId, mind, *interfaces):
        """The avatar for this user."""
        if resource.IResource in interfaces:
            return (resource.IResource, SimpleResource(), lambda: None)
        raise NotImplementedError()


class OAuthCheckerResource(resource.Resource):
    """A resource that verifies the request was oauth signed."""

    def render_GET(self, request):
        """Make a bit of html out of these resource's content."""
        header = request.requestHeaders.getRawHeaders("Authorization")[0]
        if header.startswith("OAuth "):
            return SAMPLE_RESOURCE
        request.setResponseCode(http.BAD_REQUEST)
        return "ERROR: Expected OAuth header not present."


class MockWebServer(BaseMockWebServer):
    """A mock webserver for the webclient tests."""

    def get_root_resource(self):
        """Get the root resource with all the children."""
        root = resource.Resource()
        root.putChild(SIMPLERESOURCE, SimpleResource())
        root.putChild(BYTEZERORESOURCE, ByteZeroResource())
        root.putChild(POSTABLERESOURCE, PostableResource())

        root.putChild(THROWERROR, resource.NoResource())

        unauthorized_resource = resource.ErrorPage(resource.http.UNAUTHORIZED,
                                                "Unauthorized", "Unauthorized")
        root.putChild(UNAUTHORIZED, unauthorized_resource)
        root.putChild(HEADONLY, HeadOnlyResource())
        root.putChild(VERIFYHEADERS, VerifyHeadersResource())
        root.putChild(VERIFYPOSTPARAMS, VerifyPostParameters())
        root.putChild(OAUTHRESOURCE, OAuthCheckerResource())

        db = checkers.InMemoryUsernamePasswordDatabaseDontUse()
        db.addUser(SAMPLE_USERNAME, SAMPLE_PASSWORD)
        test_portal = portal.Portal(SimpleRealm(), [db])
        cred_factory = guard.BasicCredentialFactory("example.org")
        guarded_resource = guard.HTTPAuthSessionWrapper(test_portal,
                                                        [cred_factory])
        root.putChild(GUARDED, guarded_resource)
        return root


class FakeReactor(object):
    """A fake reactor object."""
    qApp = "Sample qapp"


class ModuleSelectionTestCase(TestCase):
    """Test the functions to choose the qtnet or libsoup backend."""

    def test_is_qt4reactor_installed_not_installed(self):
        """When the qt4reactor is not installed, it returns false."""
        self.patch(sys, "modules", {})
        self.assertFalse(webclient.is_qt4reactor_installed())

    def test_is_qt4reactor_installed_installed(self):
        """When the qt4reactor is installed, it returns true."""
        fake_sysmodules = {"twisted.internet.reactor": FakeReactor()}
        self.patch(sys, "modules", fake_sysmodules)
        self.assertTrue(webclient.is_qt4reactor_installed())

    def assert_module_name(self, module, expected_name):
        """Check the name of a given module."""
        module_filename = os.path.basename(module.__file__)
        module_name = os.path.splitext(module_filename)[0]
        self.assertEqual(module_name, expected_name)

    def test_webclient_module_qtnetwork(self):
        """Test the module name for the qtnetwork case."""
        self.patch(webclient, "is_qt4reactor_installed", lambda: True)
        module = webclient.webclient_module()
        self.assert_module_name(module, "qtnetwork")

    def test_webclient_module_libsoup(self):
        """Test the module name for the libsoup case."""
        self.patch(webclient, "is_qt4reactor_installed", lambda: False)
        module = webclient.webclient_module()
        self.assert_module_name(module, "libsoup")


class WebClientTestCase(TestCase):
    """Test for the webclient."""

    timeout = 1

    @defer.inlineCallbacks
    def setUp(self):
        yield super(WebClientTestCase, self).setUp()
        self.ws = MockWebServer()
        self.addCleanup(self.ws.stop)
        self.base_iri = self.ws.get_iri()
        self.wc = webclient.webclient_factory()
        self.addCleanup(self.wc.shutdown)

    @defer.inlineCallbacks
    def test_request_takes_an_iri(self):
        """Passing a non-unicode iri fails."""
        d = self.wc.request(bytes(self.base_iri + SIMPLERESOURCE))
        yield self.assertFailure(d, TypeError)

    @defer.inlineCallbacks
    def test_get_iri(self):
        """Passing in a unicode iri works fine."""
        result = yield self.wc.request(self.base_iri + SIMPLERESOURCE)
        self.assertEqual(SAMPLE_RESOURCE, result.content)

    @defer.inlineCallbacks
    def test_get_iri_error(self):
        """The errback is called when there's some error."""
        yield self.assertFailure(self.wc.request(self.base_iri + THROWERROR),
                                 webclient.WebClientError)

    @defer.inlineCallbacks
    def test_zero_byte_in_content(self):
        """Test a reply with a nul byte in the middle of it."""
        result = yield self.wc.request(self.base_iri + BYTEZERORESOURCE)
        self.assertEqual(SAMPLE_JPEG_HEADER, result.content)

    @defer.inlineCallbacks
    def test_post(self):
        """Test a post request."""
        result = yield self.wc.request(self.base_iri + POSTABLERESOURCE,
                                       method="POST")
        self.assertEqual(SAMPLE_RESOURCE, result.content)

    @defer.inlineCallbacks
    def test_post_with_args(self):
        """Test a post request with arguments."""
        args = urllib.urlencode(SAMPLE_POST_PARAMS)
        iri = self.base_iri + VERIFYPOSTPARAMS + "?" + args
        headers = {
            "content-type": "application/x-www-form-urlencoded",
        }
        result = yield self.wc.request(iri, method="POST",
                                      extra_headers=headers, post_content=args)
        self.assertEqual(SAMPLE_RESOURCE, result.content)

    @defer.inlineCallbacks
    def test_unauthorized(self):
        """Detect when a request failed with the UNAUTHORIZED http code."""
        yield self.assertFailure(self.wc.request(self.base_iri + UNAUTHORIZED),
                                 webclient.UnauthorizedError)

    @defer.inlineCallbacks
    def test_method_head(self):
        """The HTTP method is used."""
        result = yield self.wc.request(self.base_iri + HEADONLY, method="HEAD")
        self.assertEqual("", result.content)

    @defer.inlineCallbacks
    def test_send_extra_headers(self):
        """The extra_headers are sent to the server."""
        result = yield self.wc.request(self.base_iri + VERIFYHEADERS,
                                       extra_headers=SAMPLE_HEADERS)
        self.assertIn(SAMPLE_KEY, result.headers)
        self.assertEqual(result.headers[SAMPLE_KEY], [SAMPLE_VALUE])

    @defer.inlineCallbacks
    def test_send_basic_auth(self):
        """The basic authentication headers are sent."""
        other_wc = webclient.webclient_factory(username=SAMPLE_USERNAME,
                                               password=SAMPLE_PASSWORD)
        self.addCleanup(other_wc.shutdown)
        result = yield other_wc.request(self.base_iri + GUARDED)
        self.assertEqual(SAMPLE_RESOURCE, result.content)

    @defer.inlineCallbacks
    def test_request_is_oauth_signed(self):
        """The request is oauth signed."""
        tsc = self.wc.get_timestamp_checker()
        self.patch(tsc, "get_faithful_time", lambda: defer.succeed(1))
        result = yield self.wc.request(self.base_iri + OAUTHRESOURCE,
                                       oauth_credentials=SAMPLE_CREDENTIALS)
        self.assertEqual(SAMPLE_RESOURCE, result.content)

    @defer.inlineCallbacks
    def test_oauth_signing_uses_timestamp(self):
        """OAuth signing uses the timestamp."""
        called = []

        def fake_get_faithful_time():
            """A fake get_timestamp"""
            called.append(True)
            return defer.succeed(1)

        tsc = self.wc.get_timestamp_checker()
        self.patch(tsc, "get_faithful_time", fake_get_faithful_time)
        yield self.wc.request(self.base_iri + OAUTHRESOURCE,
                              oauth_credentials=SAMPLE_CREDENTIALS)
        self.assertTrue(called, "The timestamp must be retrieved.")

    @defer.inlineCallbacks
    def test_returned_content_are_bytes(self):
        """The returned content are bytes."""
        tsc = self.wc.get_timestamp_checker()
        self.patch(tsc, "get_faithful_time", lambda: defer.succeed(1))
        result = yield self.wc.request(self.base_iri + OAUTHRESOURCE,
                                       oauth_credentials=SAMPLE_CREDENTIALS)
        self.assertTrue(isinstance(result.content, bytes),
                        "The type of %r must be bytes" % result.content)


class TimestampCheckerTestCase(TestCase):
    """Tests for the timestampchecker classmethod."""

    @defer.inlineCallbacks
    def setUp(self):
        """Initialize this testcase."""
        yield super(TimestampCheckerTestCase, self).setUp()
        self.wc = webclient.webclient_factory()
        self.patch(self.wc.__class__, "timestamp_checker", None)

    def test_timestamp_checker_has_the_same_class_as_the_creator(self):
        """The TimestampChecker has the same class."""
        tsc = self.wc.get_timestamp_checker()
        self.assertEqual(tsc.webclient_class, self.wc.__class__)

    def test_timestamp_checker_is_the_same_for_all_webclients(self):
        """The TimestampChecker is the same for all webclients."""
        tsc1 = self.wc.get_timestamp_checker()
        wc2 = webclient.webclient_factory()
        tsc2 = wc2.get_timestamp_checker()
        # pylint: disable=E1101
        self.assertIs(tsc1, tsc2)


class BasicProxyTestCase(SquidTestCase):
    """Test that the proxy works at all."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(BasicProxyTestCase, self).setUp()
        self.ws = MockWebServer()
        self.addCleanup(self.ws.stop)
        self.base_iri = self.ws.get_iri()
        self.wc = webclient.webclient_factory()
        self.addCleanup(self.wc.shutdown)

    def assert_header_contains(self, headers, expected):
        """One of the headers matching key must contain a given value."""
        self.assertTrue(any(expected in value for value in headers))

    @defer.inlineCallbacks
    def test_anonymous_proxy_is_used(self):
        """The anonymous proxy is used by the webclient."""
        settings = self.get_nonauth_proxy_settings()
        self.wc.force_use_proxy(settings)
        result = yield self.wc.request(self.base_iri + SIMPLERESOURCE)
        self.assert_header_contains(result.headers["Via"], "squid")

    @defer.inlineCallbacks
    def test_authenticated_proxy_is_used(self):
        """The authenticated proxy is used by the webclient."""
        settings = self.get_auth_proxy_settings()
        self.wc.force_use_proxy(settings)
        result = yield self.wc.request(self.base_iri + SIMPLERESOURCE)
        self.assert_header_contains(result.headers["Via"], "squid")

    if WEBCLIENT_MODULE_NAME.endswith(".txweb"):
        reason = "txweb does not support proxies."
        test_anonymous_proxy_is_used.skip = reason
        test_authenticated_proxy_is_used.skip = reason


class HeaderDictTestCase(TestCase):
    """Tests for the case insensitive header dictionary."""

    def test_constructor_handles_keys(self):
        """The constructor handles case-insensitive keys."""
        hd = HeaderDict({"ClAvE": "value"})
        self.assertIn("clave", hd)

    def test_can_set_get_items(self):
        """The item is set/getted."""
        hd = HeaderDict()
        hd["key"] = "value"
        hd["KEY"] = "value2"
        self.assertEqual(hd["key"], "value2")

    def test_can_test_presence(self):
        """The presence of an item is found."""
        hd = HeaderDict()
        self.assertNotIn("cLaVe", hd)
        hd["CLAVE"] = "value1"
        self.assertIn("cLaVe", hd)
        del(hd["cLAVe"])
        self.assertNotIn("cLaVe", hd)


class OAuthPlainTextTestCase(TestCase):
    """Test for the oauth signing code using PLAINTEXT."""

    oauth_sign = "PLAINTEXT"
    sample_method = "GET"
    sample_url = "http://one.ubuntu.com/"
    sample_params = {'next': 'yadda-yadda-yo', 'foo': 'naranja fanta'}
    timestamp = 1
    expected_params = {
        "oauth_timestamp": str(timestamp),
        "oauth_consumer_key": SAMPLE_CREDENTIALS["consumer_key"],
        "oauth_token": SAMPLE_CREDENTIALS["token"],
        "oauth_nonce": ANY_VALUE,
        "oauth_signature": ANY_VALUE,
    }

    @defer.inlineCallbacks
    def setUp(self):
        yield super(OAuthPlainTextTestCase, self).setUp()
        self.expected_params["oauth_signature_method"] = self.oauth_sign
        oauth_sign_plain = self.oauth_sign == "PLAINTEXT"
        self.wc = BaseWebClient(oauth_sign_plain=oauth_sign_plain)

    def parse_oauth_header(self, header):
        """Parse an oauth header into a tuple of (method, params_dict)."""
        params = {}

        method, params_string = header.split(" ", 1)
        for p in params_string.split(","):
            k, v = p.strip().split("=")
            params[k] = urllib.unquote(v[1:-1])

        return method, params

    def assert_headers_correct(self, headers):
        """Check that 'headers' match expected_params."""
        method, params = self.parse_oauth_header(headers["Authorization"])

        self.assertEqual(method, "OAuth")

        for k, expected_value in self.expected_params.iteritems():
            self.assertIn(k, params)
            if expected_value is not ANY_VALUE:
                self.assertEqual(params[k], expected_value)

    def test_build_oauth_headers(self):
        """Check that the oauth headers are properly built."""
        headers = self.wc.build_oauth_headers(self.sample_method,
                        self.sample_url, SAMPLE_CREDENTIALS, self.timestamp)
        self.assert_headers_correct(headers)

    def test_build_oauth_request(self, params=None):
        """Check that the oauth request are properly built."""
        request = self.wc.build_oauth_request(self.sample_method,
                           self.sample_url, SAMPLE_CREDENTIALS, self.timestamp,
                           parameters=params)

        self.assert_headers_correct(request.to_header())
        self.assertEqual(request.http_url, self.sample_url)
        if params is not None:
            for param, value in params.iteritems():
                self.assertIn(param, request.parameters)
                actual = request.parameters[param]
                self.assertEqual(value, actual)

    def test_build_oauth_request_with_params(self):
        """Check that the oauth request are properly with params."""
        self.test_build_oauth_request(params=self.sample_params)

    @defer.inlineCallbacks
    def test_build_signed_iri(self, params=None):
        """Check that the oauth signed iri is properly built."""
        self.patch(self.wc, 'get_timestamp', lambda: self.timestamp)
        iri = u'http://foo.bar.baz/ñandú'
        uri = self.wc.iri_to_uri(iri)
        request = self.wc.build_oauth_request(self.sample_method,
                           uri, SAMPLE_CREDENTIALS,
                           self.timestamp, parameters=params)

        result = yield self.wc.build_signed_iri(iri, SAMPLE_CREDENTIALS,
                                          parameters=params)

        scheme, netloc, path, params, query, _ = urlparse.urlparse(result)

        self.assertEqual(urlparse.urljoin(scheme + '://' + netloc, path), uri)

        expected = request.parameters
        actual = dict(urlparse.parse_qsl(query))
        for i in ('oauth_nonce', 'oauth_signature'):
            expected.pop(i)
            actual.pop(i)
        actual['oauth_timestamp'] = int(actual['oauth_timestamp'])
        self.assertEqual(expected, actual)

    def test_build_signed_iri_with_params(self, params=None):
        """Check that the oauth signed iri is properly built with params."""
        return self.test_build_signed_iri(params=self.sample_params)

    def patch_staticmethod(self, src, attr_name, replacement):
        """Alternate patch because self.patch fails restoring staticmethods."""
        original = getattr(src, attr_name)
        setattr(src, attr_name, staticmethod(replacement))
        self.addCleanup(setattr, src, attr_name, staticmethod(original))

    def test_iri_parameters_used_in_oauth_signature(self):
        """The iri parameters are used in the oauth signature."""
        from_consumer_and_token = oauth.OAuthRequest.from_consumer_and_token

        def fake_from_consumer_and_token(parameters=None, **kwargs):
            """Verify the parameters passed."""
            self.assertIn("fake_param", parameters)
            return from_consumer_and_token(
                                        parameters=parameters, **kwargs)

        self.patch_staticmethod(oauth.OAuthRequest, "from_consumer_and_token",
                                fake_from_consumer_and_token)

        self.sample_url += "?fake_param=fake_value"
        self.wc.build_oauth_request(self.sample_method, self.sample_url,
                                    SAMPLE_CREDENTIALS, self.timestamp,
                                    parameters={})


class OAuthHmacSha1TestCase(OAuthPlainTextTestCase):
    """Test for the oauth signing code using HMAC-SHA1."""

    oauth_sign = "HMAC-SHA1"
