# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""A webclient backend that uses twisted.web.client."""

import base64

from StringIO import StringIO

from twisted.internet import defer, protocol
from zope.interface import implements

from ubuntu_sso.utils.webclient.common import (
    BaseWebClient,
    HeaderDict,
    Response,
    UnauthorizedError,
    WebClientError,
)


class StringProtocol(protocol.Protocol):
    """Hold the stuff received in a StringIO."""

    # pylint: disable=C0103
    def __init__(self):
        """Initialize this instance."""
        self.deferred = defer.Deferred()
        self.content = StringIO()

    def dataReceived(self, data):
        """Some more blocks received."""
        self.content.write(data)

    def connectionLost(self, reason=protocol.connectionDone):
        """No more bytes available."""
        self.deferred.callback(self.content.getvalue())


class StringProducer(object):
    """Simple implementation of IBodyProducer."""

    # delay import, otherwise a default reactor gets installed
    from twisted.web import iweb

    implements(iweb.IBodyProducer)

    def __init__(self, body):
        """Initialize this instance with some bytes."""
        self.body = body
        self.length = len(body)

    # pylint: disable=C0103
    def startProducing(self, consumer):
        """Start producing to the given IConsumer provider."""
        consumer.write(self.body)
        return defer.succeed(None)

    def pauseProducing(self):
        """In our case, do nothing."""

    def stopProducing(self):
        """In our case, do nothing."""


class WebClient(BaseWebClient):
    """A simple web client that does not support proxies, yet."""

    # delay import, otherwise a default reactor gets installed
    from twisted.internet import reactor
    from twisted.web import client, http, http_headers

    # Undefined variable 'http_headers', 'client', 'reactor', 'http'
    # pylint: disable=E0602

    @defer.inlineCallbacks
    def request(self, iri, method="GET", extra_headers=None,
                oauth_credentials=None, post_content=None):
        """Get the page, or fail trying."""
        uri = self.iri_to_uri(iri)

        if extra_headers:
            headers = dict(extra_headers)
        else:
            headers = {}

        if oauth_credentials:
            timestamp = yield self.get_timestamp()
            oauth_headers = self.build_oauth_headers(method, uri,
                                                oauth_credentials, timestamp)
            headers.update(oauth_headers)

        if self.username and self.password:
            auth = base64.b64encode(self.username + ":" + self.password)
            headers["Authorization"] = "Basic " + auth

        try:
            request_headers = http_headers.Headers()
            for key, value in headers.items():
                request_headers.addRawHeader(key, value)
            agent = client.Agent(reactor)
            if post_content:
                body_producer = StringProducer(post_content)
            else:
                body_producer = None
            agent_response = yield agent.request(method, uri,
                                                 headers=request_headers,
                                                 bodyProducer=body_producer)
            raw_headers = agent_response.headers.getAllRawHeaders()
            response_headers = HeaderDict(raw_headers)
            if method.lower() != "head":
                response_content = yield self.get_agent_content(agent_response)
            else:
                response_content = ""
            if agent_response.code == http.OK:
                defer.returnValue(Response(response_content, response_headers))
            if agent_response.code == http.UNAUTHORIZED:
                raise UnauthorizedError(agent_response.phrase,
                                        response_content)
            raise WebClientError(agent_response.phrase, response_content)
        except WebClientError:
            raise
        except Exception as e:
            raise WebClientError(e.message, e)

    def get_agent_content(self, agent_response):
        """Get the contents of an agent response."""
        string_protocol = StringProtocol()
        agent_response.deliverBody(string_protocol)
        return string_protocol.deferred

    def force_use_proxy(self, settings):
        """Setup this webclient to use the given proxy settings."""
        # No proxy support in twisted.web.client
